﻿using System.Text;

using anbs_cp.Interfaces;

namespace anbs_cp.Classes.Encrypt;

/// <summary>
/// Ключ шифровки / расшифровки
/// </summary>
public sealed class EncryptKey: IEncryptKey
{
    /// <summary>
    /// Конструктор по умолчанию
    /// </summary>
    /// <param name="key">Ключ в формате массива <see cref="byte"/></param>
    public EncryptKey (byte[] key) => Key = key;

    /// <summary>
    /// Конструктор, принимающий строку и преобразующий в нужный формат
    /// </summary>
    /// <param name="utf8Key">Ключ в формате <see cref="string"/> и в кодировке utf8</param>
    public EncryptKey (string utf8Key) => Key = Encoding.UTF8.GetBytes(utf8Key);

    /// <summary>
    /// Ключ
    /// </summary>
    public byte[] Key { get; set; }

    /// <summary>
    /// Получение ключа по умолчанию
    /// </summary>
    /// <returns><see cref="EncryptKey"/></returns>
    public static EncryptKey GetDefault () => new(Enumerable.Range(0, 32).Select(static x => (byte)x).ToArray());
}