﻿using System.Security.Cryptography;
using System.Text;

namespace anbs_cp.Classes.Encrypt;

/// <summary>
/// Класс для шифровки строк
/// </summary>
public static class StringEncrypt
{
    /// <summary>
    /// Метод для шифрования строки <paramref name="text"/> с помощью ключа <paramref name="key"/>
    /// </summary>
    /// <param name="text">Строка, которая должна быть зашифрована</param>
    /// <param name="key">Ключ шифрования (в случае <value>null</value>)</param>
    /// <returns>Этот статический метод возвращает зашифрованную строку <paramref name="text"/> с помощью ключа шифрования <paramref name="key"/></returns>
    public static string Encrypt (string text, EncryptKey? key = null)
    {
        using Aes aes = Aes.Create();
        aes.Key = key?.Key ?? EncryptKey.GetDefault().Key;
        using MemoryStream ms = new();
        ms.Write(aes.IV);
        using (CryptoStream cs = new(ms, aes.CreateEncryptor(), CryptoStreamMode.Write, true))
            cs.Write(Encoding.UTF8.GetBytes(text));

        return Convert.ToBase64String(ms.ToArray());
    }

    /// <summary>
    /// Метод для дешифрования строки <paramref name="text"/> с помощью ключа <paramref name="key"/>
    /// </summary>
    /// <param name="text">Строка, которая должна быть дешифрована</param>
    /// <param name="key">Ключ шифрования (в случае <value>null</value>)</param>
    /// <returns>Этот статический метод возвращает дешифрованную строку <paramref name="text"/> с помощью ключа шифрования <paramref name="key"/></returns>
    public static string Decrypt (string text, EncryptKey? key = null)
    {
        using MemoryStream ms = new(Convert.FromBase64String(text));
        byte[] iv = new byte[16];
        int _ = ms.Read(iv);
        using Aes aes = Aes.Create();
        aes.Key = key?.Key ?? EncryptKey.GetDefault().Key;
        aes.IV = iv;
        using CryptoStream cs = new(ms, aes.CreateDecryptor(), CryptoStreamMode.Read, true);
        using MemoryStream output = new();
        cs.CopyTo(output);
        return Encoding.UTF8.GetString(output.ToArray());
    }
}